import acrort

from . cmdline_parser import ArgumentsProcessor, ArgumentParser


TAG_CONNECTION = 'connection'
TAG_SERVICE = 'service'
TAG_HOSTNAME = 'hostname'
TAG_USERNAME = 'username'
TAG_PASSWORD = 'password'


class ConnectionArgumentsProcessor(ArgumentsProcessor):
    def create_arguments_parser(self):
        parser = ArgumentParser(add_help=False)
        parser.add_argument('--service', nargs=1, choices=acrort.connectivity.SUPPORTED_PROTOCOLS, required=True)
        parser.add_argument('--connection', nargs=3, metavar=('HOST', 'USERNAME', 'PASSWORD'))
        return parser

    def process_argumens(self, config, args):
        output = {}
        output[TAG_SERVICE] = args.service[0]
        if args.connection is not None:
            output[TAG_HOSTNAME] = args.connection[0]
            output[TAG_USERNAME] = args.connection[1]
            output[TAG_PASSWORD] = args.connection[2]
        config[TAG_CONNECTION] = output


def connect_to_service(config, *, queue=None, printer=None):
    conn_config = config[TAG_CONNECTION]
    service = conn_config[TAG_SERVICE]
    if TAG_HOSTNAME not in conn_config:
        if printer is not None:
            printer.write("Connecting locally to {} ...".format(service), end='\n')
        args = (service,)
    else:
        hostname = conn_config[TAG_HOSTNAME]
        username = conn_config[TAG_USERNAME]
        password = conn_config[TAG_PASSWORD]
        cred = acrort.common.Credentials(username, password)
        if printer is not None:
            printer.write("Connecting to {} on {} ...".format(service, hostname), end='\n')
        args = (service, hostname, cred)
    try:
        connection = acrort.connectivity.Connection(*args, queue=queue)
    except acrort.Exception as exception:
        error = exception.to_error()
        if printer is not None:
            printer.write(error, end='\n')
        return (None, error)
    if printer is not None:
        printer.write('Connected as {}'.format(connection.user_name), end='\n')
    return (connection, acrort.common.SUCCESS)
