"""
@file
@brief    
@details  Copyright (c) 2001-2014 Acronis
@author   Bulat Gaifullin (bulat.gaifullin@acronis.com)
@since    $Id: _backend.py 986456 2014-12-10 15:36:21Z mmakarov $
"""

try:
    import pyastorage_client
except ImportError:
    from . import pyastorage_client

import logging

__all__ = ["Error", "connect", "CertificateStorage"]


Error = pyastorage_client.Error


def connect(host, port=None, **kwargs):
    if port is None:
        host, _, port = host.partition(':')

    return pyastorage_client.connect(host, int(port), **kwargs)


class CertificateStorage(pyastorage_client.CertificateStorage):
    def __init__(self, certificate_authority, account_id, subaccount_id=None):
        self.certificate_authority = certificate_authority
        self.account_id = account_id
        self.subaccount_id = subaccount_id

    def get(self):
        try:
            return self.certificate_authority.issue(self.account_id, self.subaccount_id)
        except Exception as e:
            logging.exception('Failed to get new certificate for {0}-{1}: {2!r}'.format(self.account_id, self.subaccount_id, e))
            raise

    def renew(self, reason):
        try:
            self.certificate_authority.invalidate(reason, self.account_id, self.subaccount_id)
        except Exception as e:
            logging.exception('Failed to renew certificate for {0}-{1}: {2!r}'.format(self.account_id, self.subaccount_id, e))
            raise
