"""
@file
@brief    Some utilities for astorage
@details  Copyright (c) 2001-2014 Acronis
@author   Bulat Gaifullin (bulat.gaifullin@acronis.com)
@since    $Id: utility.py 986456 2014-12-10 15:36:21Z mmakarov $
"""

from collections import deque


__all__ = ["traverse_subaccounts", "traverse_files", "traverse_directory"]


def _traverse_directory(directory, file_handler, is_dir, open_dir, iter_dir, recursive=True, dir_handler=None):
    file_iter = iter_dir(directory)
    dir_stack = deque()
    while directory is not None:
        recursion = False
        for client_file in file_iter:
            if is_dir(directory, client_file):
                found_dir = open_dir(directory, client_file)
                if dir_handler is not None:
                    dir_handler(found_dir)

                if not recursive:
                    continue

                dir_stack.append((directory, file_iter))
                directory = found_dir
                file_iter = iter_dir(directory)

                recursion = True
                break

            file_handler(directory, client_file)

        if not recursion:
            try:
                directory, file_iter = dir_stack.pop()
            except IndexError:
                directory, file_iter = None, None


def traverse_directory(directory, recursive=True, file_handler=None, dir_handler=None):
    return _traverse_directory(
        directory,
        file_handler=file_handler,
        is_dir=lambda _, file_record: file_record.FileType == 'FILE_DIRECTORY',
        open_dir=lambda d, file_record: d.OpenDir(file_record.FileName),
        iter_dir=iter,
        recursive=recursive,
        dir_handler=dir_handler)


def traverse_subaccounts(account_root, subaccount_handler):
    """
    traverse all subaccounts
    :param account_root: account home directory
    :param subaccount_handler: callback, that is called for each subaccount (name)
    :return: None
    """
    return traverse_directory(account_root, recursive=False, dir_handler=subaccount_handler)


def traverse_files(account_root, file_handler):
    """
    traverse all account files
    :param account_root: account home directory
    :param file_handler: callback, that is called for each file, (directory, file_record)
    :return: None
    """
    return traverse_directory(account_root, recursive=True, file_handler=file_handler)
