"""
@file
@brief    
@details  Copyright (c) 2001-2013 Acronis
@author   Bulat Gaifullin (bulat.gaifullin@acronis.com)
@since    $$Id: api.py 910972 2014-04-17 04:34:35Z bgaifullin $$
"""

from .processors import HTTPBaseAuthProcessor, HTTPRedirectProcessor
from acronis.lib import json
from urllib.request import build_opener, Request
from urllib.error import HTTPError
from urllib.parse import urljoin, quote, unquote, urlsplit, urlunsplit, splitvalue


__all__ = ['get', 'post', 'delete', 'head', 'put', 'HTTPError', 'urljoin', 'quote', 'unquote',
           'urlsplit', 'urlunsplit', 'splitvalue']


def _make_request(url, method, data=None, headers=None):
    request = Request(url, method=method, data=data)
    if isinstance(data, (dict, tuple, list)):
        request.data = json.dumps(data).encode('utf8')
        request.add_header('Content-Type', 'application/json; charset=utf-8')
    elif isinstance(data, bytes):
        request.add_header('Content-Type', 'application/octet-stream')
    elif isinstance(data, str):
        request.add_header('Content-Type', 'application/text-plain; charset=utf-8')
        request.data = data.encode('utf8')
    if headers:
        for k, v in headers.items():
            request.add_header(k, v)
    return request


def _convert_response(response):
    if response.headers.get('Content-Type', '').partition(';')[0].strip() == 'application/json':
        return json.loads(response.read().decode('utf8'))
    return response.read()


def do_request(url, method='GET', data=None, processors=None, username=None, password=None, headers=None):
    if processors is None:
        processors = [HTTPRedirectProcessor()]
    if username is not None and password is not None:
        processors.append(HTTPBaseAuthProcessor(username, password))

    processors = set(processors)
    opener = build_opener(*processors)
    return _convert_response(opener.open(_make_request(url=url, method=method, data=data, headers=headers)))


def delete(url, **kwargs):
    return do_request(url, 'DELETE', **kwargs)


def get(url, **kwargs):
    return do_request(url, 'GET', **kwargs)


def head(url, **kwargs):
    return do_request(url, 'HEAD', **kwargs)


def post(url, **kwargs):
    return do_request(url, 'POST', **kwargs)


def put(url, **kwargs):
    return do_request(url, 'PUT', **kwargs)
