"""
@file
@brief    Template builder
@details  Copyright (c) 2001-2014 Acronis
@author   Georgy Kirichenko (georgy.kirichenko@acronis.com)
@since    $$Id: template.py 944857 2014-08-15 13:08:46Z georgy.kirichenko $$
"""
from .translations import get_functions
from jinja2.utils import LRUCache
import datetime
import jinja2

__all__ = ['get_loader']


__loaders_cache = LRUCache(10)


class _Template:
    def __init__(self, template, translation):
        self.__impl = template
        self.__translation = translation

    def substitute(self, **kwargs):
        if self.__translation:
            kwargs.update(get_functions(self.__translation))
        return self.__impl.render(kwargs)


class _Loader(jinja2.Environment):
    def __init__(self, search_path, translations=None, globals=None):
        super().__init__(extensions=['jinja2.ext.i18n'], loader=jinja2.FileSystemLoader(search_path), auto_reload=False)
        self.install_null_translations()
        self.globals.update(datetime=datetime)
        self.globals.update(globals or {})
        self.__translations = translations or {}

    def load(self, name, language=None):
        return _Template(super().get_template(name), self.__translations.get(language))


def get_loader(search_path, translations=None, globals=None):
    key = search_path if isinstance(search_path, str) else tuple(search_path)

    loader = __loaders_cache.get(key)
    if loader is None:
        __loaders_cache[key] = loader = _Loader(search_path, translations, globals)
    return loader
